REBOL [
	Title: "Image Viewer"
	File: %viewer.r
	Date: 23-Jan-2000
	Purpose: {Shows jpeg, gif, bmp from a directory.}
]

view-size: 800x600

types: [%.bmp %.jpg %.jpeg %.gif]

path: to-file ask "Directory? (example: /c/images) "
if zero? length? trim path [path: %./]
if (last path) <> #"/" [append path #"/"]

last-file: none

next-image: func [files /begin /local suffix start] [
	start: files
	if not begin [files: next files]
	while [not tail? files] [
		if find types find/last first files "." [return files]
		files: next files
	]
	start
]

back-image: func [files /local suffix start] [
	start: files
	until [
		files: back files
		if find types find/last first files "." [return files]
		head? files
	]
	start
]

photo: make face [
	offset: 0x0
	size: view-size
	body: make body [image: none effect: [image]]
	feel: make feel [
		redraw: func [face n] [
			n: load join path first files
			if image? n [face/body/image: n]
			recycle
		]
	]
]

title: make face [
	size: 200x20
	offset: 10x-6 + (view-size * 0x1) - (size * 0x1)
	text: "file"
	font: make font [color: 250.250.0]
	body: make body [color: 50.50.50]
	feel: make feel [
		redraw: func [face n] [face/text: first files]
	]
]

sizer: make title [
	offset: offset + (size * 1x0) + 4x0
	size: 64x20
	font: make font [color: 0.250.250]
	feel: make feel [
		redraw: func [face n] [face/text: size? join path first files]
	]
]

next-btn: make face [
	size: 40x20
	offset: view-size - size - 6x6
	text: "Next"
	body: make body [color: 240.140.40]
	edge: make edge [effect: 'bezel]
	feel: make feel [
		engage: func [face action evt index][
			if action = 'down [
				files: next-image files
				show photo show title show sizer
			]
		]
	]
]

back-btn: make next-btn [
	offset: offset - (size * 1x0) - 2x0
	text: "Back"
	feel: make feel [
		engage: func [face action evt index][
			if action = 'down [
				files: back-image files
				show photo show title show sizer
			]
		]
	]
]

if not exists? path [print [path "does not exist"] halt]
files: next-image/begin read path
either find types find/last first files "." [
	view/size reduce [photo title sizer next-btn back-btn] view-size
][print "No image files found" halt]
