REBOL [
	Title: "CID Tutorial"
	Date: cvs-date "$Date: 2000/01/29 09:40:47 $"
	Version: cvs-version "$Revision: 1.10 $"
	Author: "Carl Sassenrath"
	File: %cidinfo.r
]

;do %/e/contra/src/cid.r

window-size: 640x480

layit: func [blk] [layout window-size blk]

stylize [
    heading title [font: [size: 24 color: 255.255.100]]
    banner image [
        size: 440x50
        font: [size: 32 shadow: 3x3]
        body: [color: 160.40.40]
    ]
    example text [
        font: [color: 255.180.100 name: "courier new"] 
        para: [origin: 30x0 wrap?: off]
        after: [
            trim/tail trim/auto text
            state: size-text self
            size/y: max size/y second state
        ]
    ]
    italic text [font: [style: [bold italic]]]
    word italic [font: [color: 255.255.100]]
    def text [size: 300x24]
]

common: layit [
    backtile %carlwaves.jpg 10.10.180
    at 500x0 image %carlwaves.jpg 160.80.40 140x480
    at 520x30 subtitle "Sections" 100x24 240.140.40 center
    space 0x4 
    button "Purpose"    [view s0]
    button "Overview"   [view s1]
    button "Example"    [view s2]
    button "Styles"     [view s3]
    button "Attributes" [view s4]
    button "Layout"     [view s5]
    button "Text"       [view s6]
    button "Images"     [view s7]
    button "Backdrops"  [view s8]
    button "Buttons"    [view s9]
    button "Fields"     [view s10]
    pad 30
    button "Feedback"   [view sc]
    button "Quit"       [either demo-object/next-file [demo-object/stop: true][quit]]
    space 0x10
]

s0: layit [
    with common
    banner "REBOL/View Tutorial"
    heading "Consumer Interface Dialect"

    subtitle "Purpose"
        text {
        The purpose of this presentation is to provide a quick
        introduction to developing interactive content through
        the consumer interface dialect (CID).  This is just one
        of many user interface dialects.  With CID, in just about
        ten minutes you will known enough to create consumer-oriented
        content on your own.
        }

    subtitle "How to Proceed..."
        text {
        The menu on the right provides navigation to each
        section of this tutorial. Just click on a button.
        }

    subtitle "Questions"
        text {
        If you have questions or problems with this tutorial, send
        them directly to us by clicking on the feedback button within
        the menu.  We're eager to hear from you.
        }

    pad 30x10 italic "-- Carl Sassenrath, Founder" 70.240.255
]

sc: layit [
    with common
    banner "Send Us Feedback"
    text {Type your comment in the box below and press
    send.  Be sure that your email settings have been 
    provided to REBOL in advance.}
    fdback: area 400x200
    across space 10x10
    button "Send" [
        either system/user/email [
            msg-snd/text: "Sending..." show msg-snd
            if not fdback/text [fdback/text: form fdback/text]
            send carl@rebol.com append fdback/text
                system/version
        ][
            msg-snd/text: "ERROR: email not configured!"
            show msg-snd
        ]
    ]
    button "Clear" [clear fdback/text show fdback]
    button "Cancel" [view s0]
    at 20x400
    msg-snd: text "" 300x24 255.50.50  ;!!! error when missing text!
]

s1: layit [
    with common
    heading "REBOL/View Overview"
        text {
        REBOL/View combines the networking strengths of REBOL/Core
        with a powerful method of creating graphical interfaces. Its
        compositing engine allows an extremely wide variety of user
        interface designs, including special custom gadgets with
        their own look and feel.  Like the rest of REBOL, simple
        things are simple to do and your interactive content
        remains small and portable (machine independent).
        }

    subtitle "Two Approaches"
        text {
        In REBOL there are two ways to build graphical interfaces.
        The easiest is to use a dialect that accepts simple
        descriptions and converts them to their underlying graphical
        objects.  We've provided the Consumer Interface Dialect to
        let you create the style of content that is typical for
        consumer presentation.  Other dialects will also become
        available, and you can always write your own as needed.
        } text {
        The other approach to building interfaces is to define the
        necessary graphical objects directly. This will be covered
        in a separate presentation.
        } text {
        Both user interface methods have advantages. For most
        applications, using a dialect produces the quickest results
        with the least effort.
        }
]

demo-text: {
demo-screen: layout 640x480 [
    backdrop %carlwaves.jpg
    title "My Example Screen"
    text "This is a simple example."
    data: area "REBOL is ..."
    across
    button "Send"  [print "sending"]
    button "Clear" [clear data/text show data]
]
}

s2: layit [
    with common
    heading "Short Example"

    subtitle "1. Layout Your Screen"
        text {You specify the styles and attributes of the
        objects that you want to put on the screen.  You can
        include text, images, buttons, backdrops and many other
        types of objects.}
        example demo-text
        text {As you see, a layout is simply a series of styles.}

    subtitle "2. Display Your Screen"
        text {To view the screen, all you need to do is:}
        example {view demo-screen}
        text {You can have any number of such screens.}
        at 300x440
        button "Click Here to View It" 160.80.80 160x24 [
            dt: do demo-text
            append dt layit [
                at 500x440
                button "Return" 160.80.80 [view s2]
            ]
            view dt
        ]
]

s3: layit [
    with common
    heading "Item Styles"
        text {Each screen is composed of a number of items
        that are of a particular style. There are a number of
        predefined styles:}
    example {
        TITLE - a title line
        SUBTITLE - a subtitle line
        TEXT - normal text
        LABEL - text used as a label
        IMAGE - a graphic image
        FRAME - image with a border
        BACKDROP - an image for the screen
        BACKTILE - a tiled screen image
        BUTTON - a click button
        TOGGLE - button that's on or off
        FIELD - single line text entry
        AREA - multi-line text entry
    }
    text {In addition, you can create your own special styles.
    However, to do so requires more detailed knowledge of
    REBOL face objects.}
    example {
        stylize [
            big-text text [font: [size: 40]]
            red-button button [body: [color: 200.5.5]]
        ]
    }
    text {The new style BIG-TEXT comes from the style TEXT
    with its font size increased to 40 point. The RED-BUTTON
    is the same as a BUTTON, but is red in color.}
]

s4: layit [
    with common
    heading "Item Style Attributes"
        text {As each style is specified, it may include
        any of these attributes.}

    ;-- !!! Make this easier to layout...
    gp1: across
    word "Text" 50x24
    def {The textual contents to be displayed. The string
    used for text areas, buttons, fields, etc.}

    guide gp1 pad 0x40
    word "Size"  50x24
    def {The width and height of the button as a pair.
    For instance, 100x24 would be 100 wide and 24 high.}

    guide gp1 pad 0x40
    word "Color"  50x24
    def {The color of the text, button, or image.  Written
    as an RGB value, as in 240.140.40 for orange.}

    guide gp1 pad 0x40
    word "Image"  50x24
    def {Filename of a graphic (JPEG, GIF, or BMP) to be
    used for images, backdrops, and buttons.  Must have
    a % to show that it's a filename.}

    guide gp1 pad 0x50
    word "Action"  50x24
    down
    def {An action block to take when user clicks on buttons, 
    hot text, or images.  Must be a block of REBOL code.}
    
    guide gp1
    text "Examples:"
    example {
        text "A Line" 200.0.0
        button "Press" 40x30 [print "Pressed"]
        image %ukiah.jpg
    }

    text {In addition, some styles will accept other attributes
    following those above. For instance, text can be CENTER, 
    LEFT, or RIGHT.  A button can have ALT strings to be
    displayed when it is selected.} 

]

layout-example: {
    ORIGIN 100x100
    BUTTON "Hello"
    BUTTON "WORLD"
    PAD 20  TEXT "Ukiah"
    INDENT 30  TEXT "Calpella"
    ACROSS BUTTON "1" BUTTON "2"
}

s5: layit [
    with common
    heading "Screen Layout"
    text {The order of items within the layout block
    determines how they are positioned on the screen.
    If you specify an image then a button, the image
    will come first, followed by the button.  The layout
    of the screen starts at the top left and proceeds
    downward.  Spacing is automatic, depending on the
    sizes of items.}
    text {In addition, you can specify the spacing between
    items with an XY pair such as 100x20. There are various
    words to control this:}
    example {
        PAD - add space before next item
        INDENT - space toward the right
        SPACE - set the auto spacing
        ACROSS - auto layout to the right
        DOWN - auto layout downward
        AT - put item at given position
        ORIGIN - origin position
    }
    text "For example:"
	example layout-example
    at 380x440
    button "View It" 160.80.80 [view-it layout-example s5]
] 

paragraph-example: {This is a longer paragraph that spans
several lines. It is placed in a separate word called
paragraph-example to take less space in the code.}

text-example: {
    title "Title Text"
    subtitle "Subtitle Text"

    text "White, Left"
    text "Green, Center" 600x24 0.255.0 center
    text "Red, Right" 600x24 255.0.0 right

    text "Hot Text" [print "hot text"]
    text "Blue Hot" 0.0.255 [print "blue hot"]

    text paragraph-example
    text paragraph-example 255.80.150 right 
}

s6: layit [
    with common
    heading "Text Styles"
    text {The text, title, and subtitle styles are predefined.
    These can be colored or aligned in various ways.  Text
    that has an action block will become hot text automatically.}

    example text-example
    text {You can define additional styles when you require
    other fonts, bold, italic, sizes, etc.}
    example {
        stylize [
            italic text [font: [style: [italic]]
            text14 text [font: [size: 14]]
        ]
    }
    at 380x440
    button "View It" 160.80.80 [view-it text-example s6]
]

showit: func [txt][istyle/text: txt show istyle]

s7: layit [
    with common
    heading "Image Styles"
    image %bay.jpg [showit "image %bay.jpg"]
    hr1: mark
    image %bay.jpg 240.125.60 [showit "image %bay.jpg 240.125.60"]
    across
    image %bay.jpg 90x50 [showit "image %bay.jpg 90x50"]
    image %bay.jpg 90x50 180.170.160 [showit "image %bay.jpg 90x50 180.170.160"]
    frame %bay.jpg 240x50 150.50.180 "Text Over the Image" [
        showit {frame %bay.jpg 240x50 150.50.180 "Text Over the Image"}] edge 0.100.200
    at 220x55
    down
    text 240x24 {Images are easy to specify. Click on each
    image to see the code that created it. You can use either
    the IMAGE style or the FRAME style.  The FRAME style places
    a border around the edge of the image.}
    text 240x24 {A color can be provided to colorize the image
    and the size of the image can be scaled.}
    at (hr1 + 205x0)
    image %bay.jpg [showit "image %bay.jpg hflip"] hflip
    at 20x440
    across
    Label "Code:"
    istyle: example "" 240.140.80 440x24
]

view-it: func [example scene /local dt][
    dt: layit load example
    append dt layit [
        at 500x440
        button "Return" 160.80.80 [view scene]
    ]
    view dt
]

bx1: {backdrop %carlwaves.jpg}
bx2: {backdrop %carlwaves.jpg 0.150.0}
bx3: {backtile %carlwaves.jpg}
bx4: {backtile %carlwaves.jpg 150.0.0}

s8: layit [
	with common
	heading "Backdrop Images"
	text {Images can be placed anywhere on the screen and scaled
	to any size.  Quite often you will want to provide an image
	that serves as a backdrop for the entire screen.  The
	BACKDROP and BACKTILE styles have been provided for this
	purpose.}
	text {The BACKDROP style takes an image and scales it to
	the size of the entire screen.  You can also colorize it.}
	xp: across
	example bx1 300x24 button "View" 160.80.80 [view-it bx1 s8]
	at (xp + 0x30)
	example bx2 300x24 button "View" 160.80.80 [view-it bx2 s8]
	at (xp + 0x60) down
	italic {Note: this image may look odd, because it is small
	and is being blown up to the full size of the screen.}
	text {The BACKTILE style will tile the image over the entire
	screen. It can also be colorized.}
	xp2: across
	example bx3 300x24 button "View" 160.80.80 [view-it bx3 s8]
	at (xp2 + 0x30)
	example bx4 300x24 button "View" 160.80.80 [view-it bx4 s8]
	at (xp2 + 0x60) down
]		

s9: layit [
	with common
	heading "Buttons"
	text {You can create a wide variety of buttons from the
	standard button styles that are provided.  In addition
	you can define new sytles and behaviors for buttons
	as you require.}
	;Need table sytle here:
	button "Simple Button"
	button "Red Button" 200.50.50
	button "Image" %carlwaves.jpg
	button "Red Image" %carlwaves.jpg 200.50.50
	button "Bay Image" %bay.jpg
	button "Big Bay Image" %bay.jpg 100x50
	toggle "Toggle" alt "Down"
	toggle "Up" %carlwaves.jpg 0.200.0 alt "Down"
	at 130x160
	text "Here's the code:"
	example {
	button "Simple Button"
	button "Red Button" 200.50.50
	button "Image" %carlwaves.jpg
	button "Red Image" %carlwaves.jpg 200.50.50
	button "Bay Image" %bay.jpg
	button "Big Bay Image" %bay.jpg 100x50
	toggle "Toggle" alt "Down"
	toggle "Up" %carlwaves.jpg 10.200.10 alt "Down"
	}
]

s10: layit [
	with common
	heading "String Fields"
	text {You can create single line or multiline string
	fields with the FIELD and AREA sytles. The lines:}
	example {
		field "Single line field"
		area "Multiline text area..."
	}
	text {Would produce:}
	slf: field "Single line field"
	area {Multiline text area...}
	pad 30
	italic {More docs to come...}
]

??: layit [
    with common
    heading "Under Construction" 400x400
    text "Come back later.."
]

view/size s0 window-size

