REBOL [
	Title: "Watch Mouse and Keyboard Events"
	File:  %events.r
	Date:  26-Jan-2000
	Purpose: {
		Watch the event handling process take place before
		your eyes.
	}
]

show-event: func [e [event!]] [
    rejoin [
        "event/type: " mold e/type newline
        "event/offset: " mold e/offset newline
        "event/char: " mold e/char newline
        "event/code: " mold e/code newline
        "event/control: " mold e/control newline
        "event/shift: " mold e/shift newline
        "event/time: " mold e/time newline
    ]
]

show-over: func [act idx] [
    rejoin [
        "action: " mold act newline
        "face offset: " mold idx
    ]
]

show-engage: func [act evt] [
    rejoin [
        "FEELING ENGAGE:" newline newline
        "action: " mold act newline
        show-event evt
	]
]

show-flash: func [face] [
    color: face/body/color
    face/body/color: 150.150.150
    show face
    face/body/color: color
	show face
]

event-list: [
	down  "MOUSE DOWN:"    mouse-down
	up    "MOUSE UP:"      mouse-up
	move  "MOUSE MOVE:"    mouse-move
	key   "KEYBOARD KEY:"  keyboard-key
]

do-events: func [/local event-port event face title loc][
    event-port: open [scheme: 'event]
    forever [
        wait event-port
        event: first event-port

		if loc: find event-list event/type [
			set [title face] reduce [second loc get third loc]
			face/text: rejoin [ 
				title newline newline show-event event]
			show-flash face
		]
		if event/type = 'quit [break]
        do event
    ]
    close event-port
    event
]

generic-face: func [os txt] [
	make face [
		offset: os
		text: txt
    	size: 200x200
    	edge: make edge [
        	size: 3x3
        	color: 80.100.80
    	]
    	body: make body [color: 200.200.200]
	]
]

mouse-down:   generic-face 0x0     "MOUSE DOWN"
mouse-up:     generic-face 200x0   "MOUSE UP"
mouse-move:   generic-face 200x200 "MOUSE MOVE"
keyboard-key: generic-face 0x200   "KEYBOARD KEY"

mouse-over: make generic-face 400x0 "FEELING OVER" [
    body: make body []
    feel: make feel [
        over: func [face act idx] [
            title: rejoin ["FEELING OVER:" newline newline]
            if act = true [
                color: face/body/color
                face/text: rejoin [title show-over act idx]
                face/body/color: 140.140.140
                show face
                face/body/color: color
            ]
            if act = false [
                face/text: rejoin [title show-over act idx]
                show face
            ]   
        ]   
    ]   
]

mouse-engage: make generic-face 400x200 "FEELING ENGAGE" [
    body: make body []
    feel: make feel [
        engage: func [face act evt] [
            switch act [
                down [
                    color: face/body/color
					face/text: show-engage act evt
                    face/body/color: 140.140.140
                    show face
                ]
                up [
					face/text: show-engage act evt
                    face/body/color: color
                    show face
                ]
                over [
					face/text: show-engage act evt
                    show face
                ]
                away [
					face/text: show-engage act evt
                    show face
                ]
            ]
        ]
    ]
]

main: make face [
    span: size: 600x400 
    pane: [
        mouse-down mouse-up mouse-over
        keyboard-key mouse-move mouse-engage
    ] 
]

view/size main 600x400

