REBOL [
	Title: "Text Fields"
	File: %textedit.r
	Date: 26-Jan-2000
	Purpose: {Allows simple text editing.}
]

edit-text: func [face evt action /local char ][
	char: evt/char
	switch/default char [
		#"^(7f)" [
			if system/view/caret-offset < length? face/text [
				remove skip face/text system/view/caret-offset
			]
		]
		#"^(BACK)" [
			if system/view/caret-offset > 0 [
				remove back skip face/text system/view/caret-offset
				system/view/caret-offset: system/view/caret-offset - 1
			]
		]
		#"^G" [
			clear face/text
			system/view/caret-offset: 0
		]
		#"^I" [
			either all [ not face/act-return
				any [ system/view/caret-offset = 0
					#"^/" = char: pick face/text system/view/caret-offset
					#"^I" = char
				]
			][
				insert skip face/text system/view/caret-offset #"^I"
				system/view/caret-offset: system/view/caret-offset + 1
			][do action]
		]
		#"^M" [
			either face/act-return [do action][
				insert skip face/text system/view/caret-offset newline
				system/view/caret-offset: system/view/caret-offset + 1
			]
		]
	][
		if char >= #" " [
			insert skip face/text system/view/caret-offset char
			system/view/caret-offset: system/view/caret-offset + 1
		]
	]
	show face
]

focus: func [face /local tmp-face][
	if system/view/focal-face [
		tmp-face: system/view/focal-face
		system/view/focal-face: none
		show tmp-face
	]
	system/view/focal-face: face
	if face [
		if not string? face/text [face/text: either face/text [form face/text][copy ""]]
		system/view/caret-offset: length? face/text
		show face
	]
]

edit: make face/feel [
	redraw: func [face index][
		face/body/color: either face = system/view/focal-face [255.255.100][
			255.255.255]
	]
	engage: func [face act event index][
		switch act [
			down [focus face]
			up [system/view/focal-face: none show face]
			key [edit-text face event face/action]
		]
	]
]

next-focus: [
	focus either tail? next find text-faces system/view/focal-face [
		first text-faces
	][
		second find text-faces system/view/focal-face
	]
]

name-face: make face [
	offset: 100x100
	size: 200x24
	body: make body [color: 255.255.255]
	edge: make edge [effect: 'ibezel]
	font: make font [style: 'bold align: 'left]
	para: make para [wrap?: off]
	text: "Bob"
	feel: edit
	act-return: true
	action: next-focus
]

lastname-face: make name-face [
	offset: 100x140
	text: "Smith"
	body: make body []
]

area-face: make name-face [
	offset: 100x180
	size: 200x100
	text: "Description"
	font: make font [valign: 'top]
	para: make para [wrap?: on]
	body: make body []
	act-return: false
]

text-faces: reduce [name-face lastname-face area-face]

view/size text-faces 400x400


