REBOL [
	Title: "Real Goods Direct (Demo)"
	Date: 25-Jan-2000
	Rights: "Images (C) Real Goods, Script (C) REBOL"
	Warning: "Do not distribute"
]

do %database.r

mainpic: %slc4.jpg
john: %schaeffer.jpg
outcolor: 80.80.200
hicolor: 160.30.30
orange: 255.160.40
ws: 640x480
discount: 0.90

months: ["January" "February" "March" "April" "May" "June" "July"
	"August" "September" "October" "November" "December"]

stylize [
	headline title [offset: 120x20 size: 500x30 at-offset: true]
	text text [size: 440x20 font: [size: 16]]
	plain text [font: [style: none]]
;	desc text [font: [size: 12]]
	italic text [font: [style: [bold italic]]]
	fine text [font: [size: 12 style: none] para: [origin: 4x4]]
	menu text [size: 100x20 font: [color: 255.180.80 align: 'center] insert after [insert options 'size]]
	image image [font: [size: 16 style: 'bold shadow: 3x3]]
	trans image [body: [color: none effect: [key 0.0.255]]]
	subfld image [size: 80x24 body: [color: 50.120.80] font: [align: 'right] 
		set 'para make face/para [origin: 5x0 margin: 5x0]]
	itemfld subfld [size: 300x24 body: [color: 150.100.80] font: [align: 'left]]
	desc text [font: [size: 12] para: [origin: 4x10] 
		set 'body make face/body [image: -1 effect: [multiply 120.120.120]]]
]

rg-common: layout ws [
	origin 0x0
	backdrop mainpic
	origin image mainpic 640x70  outcolor crop
	origin image mainpic 100x480 outcolor crop
	origin trans %rg_logo.gif 100x70
	at 200
	menu "Specials" [show-prod]
	menu "Catalog"  [view NA]
	menu "Company"  [view NA]
	menu "Account"  [show rg-icon view NA]
	at 440
	menu "Exit"     [quit]
]

NA: layout ws [
	with rg-common
	at 100x100 title "Not Available" 500x300 center
]

rg-intro: layout ws [
	with rg-common
	headline "Products for Sustainable Living"
	at 140x100
	image john 140x140
	subtitle "Your order has been shipped..."
	text {
		Carl, I'm pleased to inform you that your order has
		been shipped and you can expect to receive it by
		Friday.  With the number of orders you
		have made over prior months, I would like to reward
		your consciousness and loyalty by offering you
		an additional 5% discount off all orders for the 
		rest of the month.                                        
		Carl, thanks for trailblazing that path to a sustainable future.
	}
	pad 20x0 italic "- John Schaeffer"
	space 4
	at 300x100 image mainpic 300x140 hicolor
	at 300x100 image "Customer Summary" 300x30 hicolor
	pad 14x0
	plain "Customer: Carl Sassenrath"
	plain "Purchased: 4 items"
	plain "5% membership discount"
	plain (reform ["5% special" pick months now/month "discount"])
]

rg-product: layout ws [
	with rg-common
	at 12x100 image john 80x80 [view rg-intro]
	headline (reform [pick months now/month "Membership Specials"])

	at 320x80 space 0x0
	prod-name: image "Name" 300x30 hicolor
	prod-info: desc "Description" 300x200

	pad 10 spot: mark
	image mainpic 300x144 30.120.30 crop 200x300
	at spot image "Other Information" 300x20 30.120.30
	prod-detail: fine "Details" 300x110

	at 110x80 image mainpic 200x350 hicolor crop 200x100
	at 116x86 prod-image: image mainpic 188x188
	pad 20x10 atx: across prod-item: text "" 160x20
	at (atx + 0x30) prod-price: text "" 160x20
	at (atx + 0x60) prod-yours: italic "" 160x20 255.255.80
	at 130x380 button "Order Item" 160x24 180.80.40 [
		calc-buy
		view rg-buy
	]
	at 110x440
	button "Next" 180.80.40 [next-item]
	button "Previous" 180.80.40 [back-item]
]

prod-fields: [
	prod-name prod-image prod-info prod-detail prod-item prod-price prod-yours
]

show-prod: func [] [
	update-prod
	view rg-product
]

update-prod: func [] [
	do first database
	prod-name/text: item
	prod-info/text: trim/lines info
	prod-detail/text: detail
	prod-item/text:  rejoin ["Item #" item-num]
	prod-price/text: reform ["List Price:" price]
	prod-yours/text: reform ["Your Price:" price * discount]
	prod-image/body/image: either exists? image [load image][load john]
	buy-qty/text: "1"
	buy-how/state: false
]

next-item: func [] [
	database: next database
	if tail? database [database: back database]
	update-prod
	foreach f prod-fields [show get f]
]

back-item: func [] [
	database: back database
	update-prod
	foreach f prod-fields [show get f]
]

bs: 80x24

calc-buy: func [][
	buy-item/text: item
	buy-inum/text: join "Catalog Item #" item-num
	buy-image/body/image: buy-bd/body/image: prod-image/body/image
	buy-price/text: price
	qty: to-integer buy-qty/text
	if qty <= 0 [qty: 1 ]
	if qty > 50 [qty: 1 ]
	buy-qty/text: form qty
	buy-subtot/text: pp: price * qty
	buy-ship/text: either buy-how/state [max $24.95 (pp / 5)][pp / 10]
	buy-tax/text: pp * 0.075
	buy-total/text: buy-ship/text + (pp * 1.075)
]

refresh-buy: func [][
	calc-buy
	show buy-qty
	show buy-subtot
	show buy-tax
	show buy-ship
	show buy-total
]

rg-buy: layout ws [
	with rg-common
	at 12x100 image john 80x80 [view rg-intro]
	headline "Real Goods Membership Purchase"

	at 120x80
	buy-bd: image john 460x360 80.60.10
	across at 140x100
	buy-image: image john 100x100
	down
	buy-item: itemfld ""
	buy-inum: itemfld ""

	at 240x170 across space 0x0
	text "Quantity:" 100x24 right
	buy-qty: field 34x24 [refresh-buy]
	pad 32 text "Price:" right
	buy-price: subfld "$0"
	down space 10x10
		
	at 380x230
	text "Subtotal:" bs right
	text "Tax:" bs right
	text "Shipping:" bs right
	text "Total:" bs right

	at 460x230
	buy-subtot: subfld "$0"
	buy-tax: subfld "$0"
	buy-ship: subfld "$0"
	buy-total: subfld "$0" 110.40.80

	at 140x230
	across
	text "Delivery by:"
	buy-how: toggle "Ground" 40.60.160 [refresh-buy] alt "Overnight" 
	at 140x265 
	fine 230x20 {Total shipping price may be less depending
	on the number of products in your final order.}

	at 160x380
	across space 32x0
	button "Purchase Now" 180.80.40 100x30 [view rg-thanks] 
	button "Add To List"  180.80.40 100x30 [view rg-thanks]
	button "Cancel Order" 180.80.40 100x30 [show-prod]
]

rg-thanks: layout ws [
	WITH rg-common
	at 12x100 image john 80x80 [view rg-intro]
	headline "Purchase Confirmed"
	at 140x100
	image john 140x140
	pad 20
	subtitle "Thank you..."
	pad 5
	text {
        Carl, your order will be delivered in the next
		five days. If you need to contact us, please send
		a message or call us at 800-762-7325.
		Everything we sell is guaranteed.  If any product
		does not meet your expectations, for any reason,
		or no reason, just return it within 90 days.
    }
	pad 20x10 italic "- John Schaeffer"
	at 360x440 across
	button "Continue" [view rg-product]
	button "Message" [view rg-msg]
]

rg-msg: layout ws [
    with rg-common
	at 12x100 image john 80x80 [view rg-intro]
	headline "Send Message to Real Goods"
	at 140x140
	space 10x10
    text {Type your message in the box below and press
    send. We'll get back to you immediately.}
    fdback: area 400x200
    msgxy: across
    button "Send" [
        either system/user/email [
            msg-snd/text: "Sending..." show msg-snd
            if not fdback/text [fdback/text: form fdback/text]
            error? try [send carl@rebol.com fdback/text]
            msg-snd/text: "" show msg-snd
        ][
            msg-snd/text: "ERROR: email not configured!"
            show msg-snd
        ]
    ]
    button "Clear" [clear fdback/text system/view/caret-offset: 0 show fdback]
    button "Cancel" [view rg-intro]
    at (msgxy + 0x40)
    msg-snd: text "" 300x24 255.50.50  ;!!! error when missing text!
]

icon: make object! [	; the order here is required!!!!!
	offset:	pair!	; where the tray window should appear
	image:	image!	; a 50x50 image that is the icon for the tray
	color:	tuple!	; a color to use for colorizing the image
	blink?:	logic!	; indicates that the icon should blink
	rate:	number!	; rate at which icon should blink
	action:	function!	; function to call when icon is clicked upon
	misc:	none	; internal value
]

reb-icon: make icon [
	offset:	30x50
	image:	load %rebicon.jpg
	color:	240.220.40
	blink?:	off
	rate:	3
	action:	func[icon][
	]
]

rg-icon: make reb-icon [
	offset:	30x50
	image:	load %rgicon.jpg
	color:	240.220.40
	blink?:	on
	rate:	2
	action:	func[icon][
		icon/blink?: false
		show icon
		view rg-intro
	]
]

show reb-icon
show rg-icon
do-events

